<?php
namespace Modules\EST\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Modules\EST\Services\AjusteService;
use Teknisa\Libs\Util\Utilities;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;
use Controllers\LogController;

class AjusteController{

    protected $AjusteService;
    protected $ValidacoesEstController;
    protected $connection;
    public $entityManager;
    protected $utilities;
    protected $LogController;

    public function __construct (AjusteService $AjusteService, ValidacoesEstoqueService $ValidacoesEstoqueService, $entityManager, Utilities $utilities, $LogController) {
        $this->AjusteService = $AjusteService;
        $this->ValidacoesEstoqueService = $ValidacoesEstoqueService;
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
        $this->logController = $LogController;
    }

    public function saveInServer ($request, $response) {
        $adjustment = $request->getParameters();
        try {
            if(count($adjustment) > 0){
                $filialLanc = $adjustment['CDFILIAL'];
            }
            $this->connection->beginTransaction();

            //cria campo para salvar erros no ajuste
            $adjustment['ERROR'] = null;
            $adjustment['ERRORTYPE'] = null;

            $this->logController->saveMasterLog();
            
            //Valida parametros de datas do estoque
            $mensagemErro = $this->validaDataLancamento($adjustment);
            if($mensagemErro){
                $adjustment['ERROR'] = $mensagemErro;
                $adjustment['ERRORTYPE'] = 1; //1 - erros de data (ajuste não é salvo no banco) 2 - erro de atualização ajuste é salvo no banco mas nao atualiza estoque.
                throw new \Exception($mensagemErro);                
            }
            if(is_null($adjustment['ERROR'])){
                //Verifica se o ajuste contém produtos
                $adjustment['PRODUCTS'] = isset($adjustment['PRODUCTS'])? $adjustment['PRODUCTS'] : $adjustment['PRODUTOS'];
                $this->AjusteService->containsProducts($adjustment['PRODUCTS']);
                
                //Gera um sequencial valido para o ajuste independente se já existente ou não.
                $adjustment['NRLANCESTQ'] = $this->AjusteService->generatedValidSequential();
                
                //Seta os campos padrões do ajuste
                $adjustment = $this->AjusteService->setDefaultColumnsForAdjustment($adjustment);
                
                //Substitui o sequencial dos produtos pelo sequencial valido do ajuste
                $adjustment['PRODUCTS'] = $this->AjusteService->setValidSequential($adjustment['NRLANCESTQ'], $adjustment['PRODUCTS']);
                
                //Verifica se todos os produtos tiveram os sequencial trocado, ainda tendo relação com o ajuste
                $this->AjusteService->checkRelationship($adjustment['NRLANCESTQ'], $adjustment['PRODUCTS']);
                
                //Seta os campos padrões do ajuste
                $adjustment['PRODUCTS'] = $this->AjusteService->setDefaultColumnsForProducts($adjustment['PRODUCTS'], $adjustment['DTLANCESTQ'], $adjustment['CDFILIMOVI']);
    
                //Salva o ajuste
                $this->AjusteService->saveAdjustment($adjustment);
                $this->entityManager->flush();
                
                //Salva os produtos do ajuste
                foreach ($adjustment['PRODUCTS'] as $product) {
                    $this->AjusteService->saveProducts($product);
                    $this->entityManager->flush();
                }
                $adjustment = $this->AjusteService->changeIsNew($adjustment);
            }
            
            //caso cdfilimovi não esteja setado significa que houve erro de validação de data do primeiro lançamento
            //porem isto nao deve impedir da rotina proseguir para o restante dos lançamentos.
            if(!isset($adjustment['CDFILIMOVI']))
                $adjustment['CDFILIMOVI'] = $adjustment['CDFILIAL'];
                
            $boolAppAtuEstq = $this->AjusteService->getParamAppAtuEstq($adjustment['CDFILIMOVI']);
            //Rotina para atualizar o estoque automticamente caso parametro seja true
            if($boolAppAtuEstq){
                if(is_null($adjustment['ERROR'])){
                    $mensagemErro = $this->atualizaEstoque($adjustment,$response);
                    if($mensagemErro){
                        $adjustment['ERROR'] = $mensagemErro;
                        $adjustment['ERRORTYPE'] = 2; //1 - erros de data (ajuste não é salvo no banco) 2 - erro de atualização ajuste é salvo no banco mas nao atualiza estoque.
                        throw new \Exception($mensagemErro);   
                    }
                }
            }
            $this->connection->commit();
            $return = array(
            					'Status'=> 'S',
            					'Mensagem'=> 'Ajuste Cadastrado com Sucesso',
            					'Pk'=> $adjustment['CDFILIAL'].$adjustment['NRLANCESTQ']
            			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
            //$response->setDataSets(array(new DataSet('adjustment', $adjustment)));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $return = array(
            					'Status'=> 'E',
            					'Mensagem'=> $e->getMessage(),
            					'Pk'=> '',
            					'ErrorType'=> isset($adjustment['ERRORTYPE'])? $adjustment['ERRORTYPE'] : null
            			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
            //$response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaDataLancamento($row) {
        try {
            $dadosValidacao = $row;
            

            $filial = $dadosValidacao['CDFILIAL'];
            $dtLancamento = $dadosValidacao['DTLANCESTQ'];
            $operacao = 'A'; //ajuste || implantacao
            $sistema = 'EST';
            $dtEmissao = $dadosValidacao['DTLANCESTQ'];

            $this->ValidacoesEstoqueService->validaDataLancamento(
                $filial, $dtLancamento, $operacao, $sistema, $dtEmissao
            );
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }
    
    public function zeraInventario($row) {
        try {
            //$this->connection->beginTransaction();
            $filial = $row['CDFILIAL'];
            $lancamento = $row['NRLANCESTQ'];
            $idordem = 'U';

            $this->ValidacoesEstoqueService->zeraInventario($filial, $lancamento, $idordem);
            
            //$this->connection->commit();
        } catch(\Doctrine\DBAL\DBALException $e) {
            return $e->getMessage();
            //$this->connection->rollback();
        } catch (\Exception $e) {
            return $e->getMessage();
            //$this->connection->rollback();
            //$response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }
    
    public function atualizaEstoque($row,$response) {
        try {
            $payload = $row;

            #gets relevant data from payload
            $cdfilial              = $payload['CDFILIAL'];
            $nrlancestq            = $payload['NRLANCESTQ'];
            $verifyIrregularities  = true; //verificar este parametro
            $abortAtIrregularities = true; //verificar este parametro
            $seqItemCodes          = null;

            #calls procedure validation, preparation and execution, receiving any irregular products, if applicable
            $irregularProducts = $this->ValidacoesEstoqueService->atualizaEstoque(
              
              
                $cdfilial, $nrlancestq, $verifyIrregularities, $abortAtIrregularities, $seqItemCodes
            );

            #sends data to frontend through response's dataset
            $response->addDataset(new DataSet("irregularProducts", $irregularProducts));
        } catch(\Doctrine\DBAL\DBALException $e) {
            #catches doctrine exceptions and prepares message for response's error
            return $e->getMessage();

            $response->setError(new Error($message, $e->getCode()));
        } catch (\Exception $e) {
            #catches generic error and sends message to frontend through response's error
            return $e->getMessage();
        }
    }
        
    public function setEntityManager($entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getEntityManager() {
        return $this->entityManager;
    }
}