<?php

/**
 * Enter description here ...
 *
 * @package Logic
 *
 * @author  Bruno_e_Luis
 *
 */
class PessoaLogic {

    const ATUALIZADO = 'Registro de Pessoa Atualizado';
    const INCLUIDO = 'Registro de Pessoa Incluido';
    const NAOCADASTRADO = 'Registro de Pessoa nao cadastrado';

    const TIPOINSCPESSOA = 'F';
    const IDATIVO = 'S';
    const CDTIPOINSC = 'CPF';
    const IDPESSOAFISICA = 'N';
    const IDINSTUICAO = 'S';

    private $_idPessoa;


    /**
     * Enter description here ...
     *
     * @param integer $nrOrgTrab       Numero da Organizacao de Trabalho.
     * @param integer $nrParceiro      Numero do Parceiro.
     * @param string  $cpf             CPF.
     * @param string  $nmPrincipalParc Nome Principal do Parceiro.
     * @param string  $nmSecundariParc Nome Secundario do Parceiro.
     * @param string  $idPessoaFisica  Identificador de Pessoa Fisica.
     * @param string  $idInstituicao   Identificador de Instituicao.
     *
     * @throws Exception $e Propaga.
     * @return void
     */
    public function incluiPessoaMestre($nrOrgTrab, $nrParceiro, $cpf, $nmPrincipalParc, $nmSecundariParc, $dtnascpessoa ){
        try {

            if (empty($nrParceiro) === true) {
                $parceiro = new ParcnegocioLogic();
                $parceiro->insereParceiroPessoa( $nrOrgTrab, $cpf, $nmPrincipalParc, $nmSecundariParc,$dtnascpessoa);
                $nrParceiro = $parceiro->getIdParceiro();
                unset($parceiro);
            }

            $pessoa = new GpepessoaEntity();

            // Verifica se existe o mestre.
            $objPessoa = $pessoa->retornaPessoaPorParceiro($nrOrgTrab, $nrParceiro);
            if (is_object($objPessoa) === false) {
                // Cria novo mestre.
                $this->_idPessoa = $pessoa->incluiPessoa($nrOrgTrab, $nrParceiro);
            } else {
                $this->_idPessoa = $objPessoa->getNrpessoa();
                // Atualiza a Pessoa.
                $objPessoa->setNrparcnegocio($nrParceiro);
                $objPessoa->persist();
            }

            unset($pessoa);
            unset($objPessoa);
        } catch(Exception $e){
            throw $e;
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrgTrab Numero da Organizacao de Tabalho.
     * @param integer $nrPessoa  Numero da Pessoa.
     *
     * @throws Exception $e Propaga.
     * @return void
     */
    public function deletaPessoa($nrOrgTrab, $nrPessoa){
        try{

            $pessoah = new GpepessoahEntity();
            $objPessoah = $pessoah->retornaHistoricoPorPessoa($nrPessoa, $nrOrgTrab);
            if (count($objPessoah) === 0) {
                // Destroi mestre.
                $pessoa = new GpepessoaEntity();
                $pessoa->destroiMestre($nrPessoa, $nrOrgTrab);

                unset($pessoa);
            }
        } catch(Exception $e){
            throw $e;
        }
        unset($pessoah);
        unset($objPessoah);

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrgTrab Numero da Organizacao de Tabalho.
     * @param integer $nrPessoa  Numero da Pessoa.
     *
     * @throws Exception $e Propaga.
     * @return void
     */
    public function deletaPessoaHistorico($nrOrgTrab, $nrPessoa){
        try{

            $conn = TkDbAdapter::getWorkConnection();
            $conn->beginTransaction();

            $pessoaH = new GpepessoahEntity();
            $pessoaH->carregaPessoasHistoricos($nrOrgTrab, $nrPessoa);
            $pessoaH->apagaPessoasHistoricos();
            $pessoaH->persist();

            $conn->commit();

            unset($pessoaH);

        } catch(Exception $e){
            $conn->rollback();
            throw new Exception('PESSOAH '.$e);
        }
    }


    /**
     * Enter description here ...
     *
     * @return Ambigous <boolean, nrpessoa, NULL>
     */
    public function getIdPessoa(){
        return $this->_idPessoa;
    }


    /**
     * Enter description here ...
     *
     * @param integer    $nrOrgTrab      Numero da Organizacao de Trabalho.
     * @param integer    $nrPessoa       Numero da Pessoa.
     * @param TkDateTime $mesCompetencia Mes de competencia.
     *
     * @return boolean
     */
    public function verificaPessoaHistorico($nrOrgTrab, $nrPessoa, TkDateTime $mesCompetencia){

        $pessoah = new GpepessoahEntity();

        $objPessoaH = $pessoah->retornaHistoricoAtualPorPessoa($nrOrgTrab, $nrPessoa, $mesCompetencia);

        if (is_object($objPessoaH) === true) {
            return true;
        }

        unset($pessoah);
        unset($objPessoaH);
        return false;
    }


    /**
     * Retorna idade da pessoa.
     *
     * @param integer $nrpessoa       Numero da Pessoa.
     * @param integer $nrorg          Numero da Organiza��o.
     * @param string  $dtmescompetenc Mes de Competencia.
     *
     * @return integer
     *
     * @author luistavares - 01/06/2011
     *         brunofeliciano - 28/06/2011 - Inclusao do parametro $dtmescompetenc.
     */
    public function getIdadePessoa($nrpessoa, $nrorg, $dtmescompetenc){

        $obj_pessoa = new GpepessoahEntity();
        $pessoa = $obj_pessoa->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $dtmescompetenc);

        if (is_object($pessoa) === true) {
            $dtnascpessoa = $pessoa->getDtnascpessoa();
            $now = new TkDateTime();
            $date = new TkDateTime($dtnascpessoa);
            $idade = $now->yearsAfter($date);
            return $idade;
        } else {
            return 0; // Retorna idade 0 se n�o possuir registro.
        }

    }


    /**
     * Remove todos os registros que se relaciona � pessoa a ser apagada.
     *
     * @param integer $nrpessoa      Nr Pessoa.
     * @param integer $nrparcnegocio Nr Parceiro Negocio.
     * @param integer $nrorg         Nr Org.
     *
     * @return boolean
     *
     * @author luistavares - 23/08/2011
     */
    public function removeCascata($nrpessoa, $nrparcnegocio, $nrorg){
        $comunicaparc = new ComunicaparcEntity();
        $lista_comunicaparc = $comunicaparc->retornaComunicaParcPorPessoa($nrparcnegocio, $nrorg);
        $comunicaparc->destroiComunicaParc($lista_comunicaparc);

        $relacionaparc = new RelacionaparcEntity();
        $lista_relacionaparc = $relacionaparc->retornaRelacionaParcPorPessoa($nrparcnegocio, $nrorg);
        $relacionaparc->destroiRelacionaParc($lista_relacionaparc);

        // Funcionando at� aqui.
        $pessoah = new GpepessoahEntity();
        $historico = $pessoah->retornaHistoricoPorPessoa($nrpessoa, $nrorg);
        // Dando Erro no persist \/.
        $pessoah->destroiPessoaH($historico);

        // Dando Erro no persist \/.
        $enderecoparc = new EnderecoparcEntity();
        $lista_enderecoparc = $enderecoparc->retornaEnderecoParcPorPessoa($nrparcnegocio, $nrorg);
        $enderecoparc->destroiEnderecoParc($lista_enderecoparc);

        return true;
    }


    /**
     * Inclui Pessoa.
     *
     * @param integer $nrpessoa      Numero da Pessoa.
     * @param integer $nrorg         Numero da Organizacao.
     * @param integer $nrparcnegocio Numero do Parceiro de Negocio.
     *
     * @return string
     *
     * @author brunofeliciano - 25/07/2011
     */
    public function incluiPessoa($nrpessoa, $nrorg, $nrparcnegocio) {
        try {

            $pessoa = new GpepessoaEntity();
            $pessoa->incluiPessoa($nrpessoa, $nrorg, $nrparcnegocio);
            return $this::INCLUIDO;

        } catch (Exception $e){
            tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
            return $this::NAOCADASTRADO;
        }
    }


    /**
     * Inclui um Hist�rico para Pessoa, caso n�o exista.
     *
     * @param integer $nrpessoa       Numero da Pessoa.
     * @param integer $nrpessoah      Numero do Historico da Pessoa.
     * @param integer $nmpessoa       Novo nome da Pessoa.
     * @param integer $nrorg          Numero da Organiza��o.
     * @param integer $dtmescompetenc Mes de Competencia.
     * @param integer $cpf            CPF.
     *
     * @return void
     *
     * @author luistavares - 19/08/2011
     */
    public function incluiPessoaH($nrpessoa, $nrpessoah, $nmpessoa, $nrorg, $dtmescompetenc, $cpf){
        $pessoah = new GpepessoahEntity();
        $obj_historico = $pessoah->retornaHistorico($nrpessoah, $nrorg);
        if (is_object($obj_historico) === false) {
            $pessoah->incluiPessoaH($nrpessoa, $nrpessoah, $nmpessoa, $nrorg, $dtmescompetenc, $cpf);
        }
    }


    /**
     * Inclui um Hist�rico para Pessoa, caso n�o exista.
     *
     * @param integer $nrpessoa       Numero da Pessoa.
     * @param integer $nrpessoah      Numero do Historico da Pessoa.
     * @param integer $nmpessoa       Novo nome da Pessoa.
     * @param integer $nrorg          Numero da Organiza��o.
     * @param integer $dtmescompetenc Mes de Competencia.
     * @param integer $cpf            CPF.
     *
     * @return string|boolean
     *
     * @author luistavares - 01/06/2011
     *         brunofeliciano - 20/06/2011 - Inclusao do parametro $dtmescompetenc.
     */
    public function incluiPessoaHOld($nrpessoa, $nrpessoah, $nmpessoa, $nrorg, $dtmescompetenc, $cpf){

        try {
            $pessoa = new GpepessoaEntity();
            $pessoa = $pessoa->retornaPessoa($nrpessoa, $nrorg);
            if (is_object($pessoa) === true) {
                $pessoah = new GpepessoahEntity();
                $obj_pessoah = $pessoah->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $dtmescompetenc);

                if (is_object($obj_pessoah) === false) {
                    /*$this->inserePessoaH(
                        $nrpessoa, $nrpessoah, $nmpessoa, $nrorg, $dtmescompetenc
                    );*/
                    $pessoah->incluiPessoaH($nrpessoa, $nrpessoah, $nmpessoa, $nrorg, $dtmescompetenc, $cpf);
                    return $this::INCLUIDO;
                }
            }

            return $this::NAOCADASTRADO;

        } catch (Exception $e){
            tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
            return $this::NAOCADASTRADO;
        }
    }


    /**
     * Inclui um Hist�rico para Pessoa, caso n�o exista. Se existir, atualiza o Hist�rico.
     *
     * @param integer $nrpessoa       Numero da Pessoa.
     * @param integer $nmpessoa       Novo nome da Pessoa.
     * @param string  $cpf            CPF.
     * @param integer $nrorg          Numero da Organiza��o.
     * @param integer $dtmescompetenc Mes de Competencia.
     *
     * @return string|boolean
     *
     * @author luistavares - 01/06/2011
     *         brunofeliciano - 05/07/2011 - Inclui historico se nao existir historico na competencia atual.
     */
    public function atualizaPessoaH($nrpessoa, $nmpessoa, $cpf, $nrorg, $dtmescompetenc){

        try {
            $pessoa = new GpepessoaEntity();
            $obj_pessoa = $pessoa->retornaPessoa($nrpessoa, $nrorg);
            if (is_object($obj_pessoa) === true) {
                $pessoah = new GpepessoahEntity();
                $obj_pessoah = $pessoah->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $dtmescompetenc);
                if (is_object($obj_pessoah) === true) {
                    if ($nmpessoa !== $obj_pessoah->getNmpessoa() || $cpf !== $obj_pessoah->getNrcpfpessoa()) {
                        // Verifica se houve alteracao.
                        if ($dtmescompetenc !== $obj_pessoah->getDtmescompetenc()) {
                            // Cria novo registro se n�o existe historico na competencia.
                            $pessoah_aux = new GpepessoahEntity();
                            $pessoah_aux->copy($obj_pessoah);

                            $nrpessoah = rand(0, 9999); /* @todo gerar novo codigo.*/

                            $pessoah_aux->setNrpessoah($nrpessoah);
                            $pessoah_aux->setNmpessoa($nmpessoa);
                            $pessoah_aux->setNrcpfpessoa($cpf);
                            $pessoah_aux->setDtmescompetenc($dtmescompetenc);
                            $pessoah_aux->persist();
                        } else {
                            // Altera registro ja existente na competencia.
                            $obj_pessoah->setNmpessoa($nmpessoa);
                            $obj_pessoah->setNrcpfpessoa($cpf);
                            $obj_pessoah->persist();
                        }

                        return $this::ATUALIZADO;
                    }
                }
            }
        } catch (Exception $e){
            tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
            return  $this::NAOCADASTRADO;
        }

        return $this::INCLUIDO;
    }


    /**
     * Valida CPF da pessoa. CPF vazio � valido.
     *
     * @param integer $cpf CPF.
     *
     * @return boolean
     *
     * @author brunofeliciano 12/05/2011
     */
    public function verificaCPF($cpf){
        $result = true;

        // CPF vazio.
        if ($cpf !== '') {
            $result = tk_validate_cpf($cpf);
        }

        return (boolean)$result;
    }


    /**
     * Verifica M�s de Compet�ncia do Historico, se existir � atualizado, caso contrario � incluido.
     *
     * @param string  $dtmescompetenc Data Mes de Competencia.
     * @param integer $nrpessoa       Numero da Pessoa.
     * @param integer $nrpessoah      Numero do Historico da Pessoa.
     * @param integer $nrorg          Numero da Organiza��o.
     *
     * @return boolean
     *
     * @author luistavares - 12/05/2011
     */
    public function verificaMesCompetencia($dtmescompetenc, $nrpessoa, $nrpessoah, $nrorg){

        try{
            $pessoa = Gpepessoah::retornaHistoricoPorMesCompetencia($nrorg, $nrpessoa, $dtmescompetenc);

            if (is_object($pessoa) === true) {
                // Existe Historico no mes de competencia cadastrado.
                $nrphcadastrado = (int) $pessoa->getNrpessoah();

                if ($nrpessoah === $nrphcadastrado) {
                    // Atualiza��o do Historico.
                    return true;
                } else {
                    // Registro deve sobrepor Historico antigo.
                    // @todo implementar metodo para sobreescrever Historico.
                    return false;
                }
            } else {
                // N�o existe Historico no mes de competencia cadastrado, registro pode ser incluido.
                return true;
            }
        } catch (Exception $e){
            tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
            return false;
        }
    }


    /**
     * Verifica se Parceiro de Negocio a ser inserido pode ser relacinado a pessoa.
     *
     * @param integer $nrparcnegocio N�mero Parceiro de Neg�cio.
     * @param integer $nrpessoa      N�mero da Pessoa.
     * @param integer $nrorg         N�mero da Organiza��o.
     *
     * @return boolean
     *
     * @author luistavares - 12/05/2011
     */
    public function verificaParcNegocio($nrparcnegocio, $nrpessoa, $nrorg){

        // Verifica se n�o existe parceiro cadastrado para pessoa.
        // Caso exista verifica se pertence a mesma pessoa.
        $pessoa = new GpepessoaEntity();
        $pessoa = $pessoa->retornaPessoaPorParceiro($nrorg, $nrparcnegocio);
        if (is_object($pessoa) === true) {
            // Existe Parceiro cadastrado.
            $nrpcadastrada = (int) $pessoa->getNrpessoa();

            if ((int)$nrpessoa === $nrpcadastrada) {
                // Parceiro pertence a mesma pessoa.
                return true;
            } else {
                // Parceiro existe porem pertence a outra pessoa.
                return false;
            }
        } else {
            // N�o existe Parceiro cadastrado.
            return true;
        }
    }


    /**
     * Inclui Parceiro de Negocio caso CPF n�o esteja cadastrado.
     *
     * @param string  $cpf      CPF.
     * @param integer $nrorg    Numero da Organizacao.
     * @param string  $nmpessoa Nome da Pessoa.
     *
     * @return integer
     *
     * @author brunofeliciano
     *         brunofeliciano - 02/09/2011 -  correcao do metodo retornaParceiroPorInscricao()
     */
    public function retornaParcNegocioPorCPF($cpf, $nrorg, $nmpessoa){

        try {
            $nrparcnegocio = 0;
            $tipoinscricao = $this::CDTIPOINSC; /* Pessoa fisica. */

            $obj_parceiro = new ParcnegocioEntity();
            $parceiro = $obj_parceiro->retornaParceiroPorInscricao($tipoinscricao, $cpf, $nrorg);

            if (is_object($parceiro) === true) {
                // Caso exista parceiro cadastrado, retorna chave existente.
                $nrparcnegocio = $parceiro->getNrparcnegocio();
            } else {
                // Caso contrario cria novo com valores default.
                $nrinscricaoparc = $cpf; // CPF.
                // $nrorg = ParametrosFacade::getNrOrgTrabalho(Ambiente::getNrorg(), Ambiente::getCdOperador());.
                /* @todo verificar strategy de parcnegocio */
                $nrparcnegocio = Novocodigo::geraCodigo($nrorg, 'PARCNEGOCIO', 10);

                $nmprincipalparc = $nmpessoa;
                $nmsegundariparc = $nmpessoa;

                // Default para pessoa fisica.
                $dtinclusao = new TkDateTime();
                $idativo = $this::IDATIVO;
                $idpessoafisica = $this::IDPESSOAFISICA;
                $idinstituicao = $this::IDINSTUICAO;

                $obj_parceiro->incluiParcNegocio(
                    $nrparcnegocio, $nrorg, $nmprincipalparc, $nmsegundariparc,
                    $nrinscricaoparc, $tipoinscricao, $idativo, $dtinclusao,
                    $idpessoafisica, $idinstituicao
                );
            }

            return $nrparcnegocio;
        } catch (Exception $e){
            tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
            return 0;
        }
    }


}